"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
}
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const types_1 = require("./types");
const _debug = debug_1.default('rpc-client');
class Client {
    /**
     * Creates an instance of Json Rpc 2.0 Client.
     * @param {Transport} transport
     * @memberof Client
     */
    constructor(transport) {
        this.transport = transport;
        this.id = 1;
    }
    /**
     * invoke remote method
     *
     * @param {string} method
     * @param {...any[]} params
     * @returns
     * @memberof Client
     */
    invoke(method, ...params) {
        return __awaiter(this, void 0, void 0, function* () {
            const reqId = this.id++;
            const payload = {
                id: reqId,
                method,
                params,
                jsonrpc: '2.0'
            };
            let jsonResponse;
            try {
                jsonResponse = yield this.transport.invoke(payload);
            }
            catch (e) {
                throw new types_1.TransportError(e);
            }
            if (!jsonResponse) {
                throw new types_1.TransportError('not return a json string');
            }
            const ret = JSON.parse(jsonResponse);
            if (!ret) {
                throw new types_1.TransportError('parse ret error');
            }
            if (ret.error) {
                throw new types_1.RemoteError(ret.error.message, ret.error.code, ret.error.data);
            }
            if (ret.id === reqId) {
                return ret.result;
            }
            else {
                throw new types_1.RemoteError('id not equal', -32600, ret);
            }
        });
    }
    /**
     * send a notify
     *
     * @param {string} method
     * @param {...any[]} params
     * @memberof Client
     */
    notify(method, ...params) {
        this.transport.notify({
            method,
            params,
            jsonrpc: '2.0'
        });
    }
    /**
     * batch call
     *
     * @memberof Client
     */
    batch() {
        const payloads = [];
        const client = this;
        return {
            invoke: function (method, ...params) {
                payloads.push({
                    id: client.id++,
                    method,
                    params,
                    jsonrpc: '2.0'
                });
                return this;
            },
            notify: function (method, ...params) {
                payloads.push({
                    method,
                    params,
                    jsonrpc: '2.0'
                });
                return this;
            },
            execute: function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const res = yield client.transport.invoke(payloads);
                    if (!res) {
                        return [];
                    }
                    else {
                        return JSON.parse(res);
                    }
                });
            }
        };
    }
    as(namespace) {
        const proxyOjbect = {
            inspect: false,
            toJSON() { return {}; },
        };
        return new Proxy(proxyOjbect, {
            get: (target, prop) => {
                if (typeof prop !== 'string') {
                    return target[prop];
                }
                if (typeof target[prop] !== 'undefined') {
                    return target[prop];
                }
                const method = namespace ? namespace + '.' + prop : prop;
                return (...args) => {
                    return this.invoke(method, ...args);
                };
            }
        });
    }
}
exports.Client = Client;
//# sourceMappingURL=client.js.map