"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("./types");
class ServerProxy {
    constructor(instance) {
        this.instance = instance;
    }
    on(event, handle) {
        this.exceptionHandle = handle;
    }
    handle(payload) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof payload === 'string') {
                payload = JSON.parse(payload);
            }
            if (Array.isArray(payload)) {
                return this.handleBatch(payload);
            }
            try {
                if (!payload) {
                    throw new types_1.ParseError('parse error: json parse error', -32700);
                }
                if (payload.jsonrpc !== '2.0') {
                    throw new types_1.ParseError('parse error: jsonrpc not equal 2.0', -32700);
                }
                if (!payload.method) {
                    throw new types_1.ParseError('Invalid Request', -32600);
                }
                if (typeof this.instance[payload.method] !== 'function') {
                    throw new types_1.ParseError('no method', -32601);
                }
            }
            catch (e) {
                return this.parseError(e);
            }
            try {
                const params = payload.params || [];
                let ret;
                if (Array.isArray(params)) {
                    ret = yield this.instance[payload.method](...params);
                }
                else {
                    ret = yield this.instance[payload.method](params);
                }
                if (payload.id !== void 0) {
                    return {
                        'id': payload.id,
                        'jsonrpc': "2.0",
                        'result': ret
                    };
                }
            }
            catch (e) {
                // if payload is a notify ignore the error
                if (payload.id) {
                    if (this.exceptionHandle) {
                        return this.exceptionHandle(payload, e) || this.makeError(payload.id, e);
                    }
                    else {
                        return this.makeError(payload.id, e);
                    }
                }
                else if (this.exceptionHandle) {
                    this.exceptionHandle(payload, e);
                }
            }
        });
    }
    handleBatch(payloads) {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = payloads.map(p => this.handle(p));
            const arr = yield Promise.all(promises);
            const vaildResults = arr.filter(r => !!r);
            //  the server MUST NOT return an empty Array and should return nothing at all.
            return vaildResults.length > 0 ? vaildResults : undefined;
        });
    }
    parseError(e) {
        return {
            'id': null,
            'jsonrpc': "2.0",
            'error': {
                'code': e.code || -32700,
                'message': e.message || 'parse error'
            }
        };
    }
    makeError(id, e) {
        return {
            'id': id,
            'jsonrpc': "2.0",
            'error': {
                'code': e instanceof types_1.RemoteError ? e.code : -32603,
                'message': e instanceof types_1.RemoteError ? e.message : 'internal error'
            }
        };
    }
}
exports.ServerProxy = ServerProxy;
/**
 * RpcServer
 *
 * @export
 * @class Server
 * @extends {EventEmitter}
 * @implements {RpcServer}
 */
class Server extends ServerProxy {
    constructor() {
        super({});
    }
    /**
     * 暴露一个方法
     *
     * @param {string} method
     * @param {Function} func
     * @memberof Server
     */
    expose(method, func) {
        this.instance[method] = func;
    }
}
exports.Server = Server;
//# sourceMappingURL=server.js.map